@@ -1,7 +1,13 @@
 #Copyright ReportLab Europe Ltd. 2000-2017
 #see license.txt for license details
-#history https://bitbucket.org/rptlab/reportlab/history-node/tip/src/reportlab/platypus/tables.py
-__version__='3.3.0'
+#history https://hg.reportlab.com/hg-public/reportlab/log/tip/src/reportlab/platypus/tables.py
+__all__= (
+        'Table',
+        'TableStyle',
+        'CellStyle',
+        'LongTable',
+        )
+__version__='3.5.21'
 
 __doc__="""
 Tables are created by passing the constructor a tuple of column widths, a tuple of row heights and the data in
@@ -60,17 +66,17 @@ class CellStyle(PropertySet):
 class TableStyle:
     def __init__(self, cmds=None, parent=None, **kw):
         #handle inheritance from parent first.
-        commands = []
         if parent:
             # copy the parents list at construction time
-            commands = commands + parent.getCommands()
+            pcmds = parent.getCommands()[:]
             self._opts = parent._opts
             for a in ('spaceBefore','spaceAfter'):
                 if hasattr(parent,a):
                     setattr(self,a,getattr(parent,a))
-        if cmds:
-            commands = commands + list(cmds)
-        self._cmds = commands
+        else:
+            pcmds = []
+
+        self._cmds = pcmds + list(cmds or [])
         self._opts={}
         self._opts.update(kw)
 
@@ -282,7 +288,7 @@ class Table(Flowable):
         self._spanCmds = []
         self._nosplitCmds = []
         self._srflcmds = []
-        # NB repeatRows can be a list or tuple eg (1,) reapesat only the second row of a table
+        # NB repeatRows can be a list or tuple eg (1,) repeats only the second row of a table
         # or an integer eg 2 to repeat both rows 0 & 1
         self.repeatRows = repeatRows
         self.repeatCols = repeatCols
@@ -333,8 +339,6 @@ class Table(Flowable):
         for row in data:
             outRow = [normCell(cell) for cell in row]
             outData.append(outRow)
-        from pprint import pprint as pp
-        #pp(outData)
         return outData
 
     def identity(self, maxLen=30):
@@ -573,7 +577,7 @@ class Table(Flowable):
                             if canv: canv._fontname, canv._fontsize, canv._leading = s.fontname, s.fontsize, s.leading or 1.2*s.fontsize
                             if ji in colSpanCells:
                                 if not span: continue
-                                w = max(colpositions[span[2]+1]-colpositions[span[0]],w)
+                                w = max(colpositions[span[2]+1]-colpositions[span[0]],w or 0)
                             dW,t = self._listCellGeom(v,w or self._listValueWidth(v),s)
                             if canv: canv._fontname, canv._fontsize, canv._leading = saved
                             dW = dW + s.leftPadding + s.rightPadding
@@ -994,20 +998,23 @@ class Table(Flowable):
             if value is None:
                 spanRects[coord] = None
             else:
-                col0, row0, col1, row1 = value
-                if row1>=rlim: continue
-                col,row = coord
-                if col1-col0>0:
-                    for _ in xrange(col0+1,col1+1):
-                        vBlocks.setdefault(colpositions[_],[]).append((rowpositions[row1+1],rowpositions[row0]))
-                if row1-row0>0:
-                    for _ in xrange(row0+1,row1+1):
-                        hBlocks.setdefault(rowpositions[_],[]).append((colpositions[col0],colpositions[col1+1]))
-                x = colpositions[col0]
-                y = rowpositions[row1+1]
-                width = colpositions[col1+1] - x
-                height = rowpositions[row0] - y
-                spanRects[coord] = (x, y, width, height)
+                try:
+                    col0, row0, col1, row1 = value
+                    if row1>=rlim: continue
+                    col,row = coord
+                    if col1-col0>0:
+                        for _ in xrange(col0+1,col1+1):
+                            vBlocks.setdefault(colpositions[_],[]).append((rowpositions[row1+1],rowpositions[row0]))
+                    if row1-row0>0:
+                        for _ in xrange(row0+1,row1+1):
+                            hBlocks.setdefault(rowpositions[_],[]).append((colpositions[col0],colpositions[col1+1]))
+                    x = colpositions[col0]
+                    y = rowpositions[row1+1]
+                    width = colpositions[col1+1] - x
+                    height = rowpositions[row0] - y
+                    spanRects[coord] = (x, y, width, height)
+                except:
+                    annotateException('\nspanning problem in %s' % (self.identity(),))
 
         for _ in hBlocks, vBlocks:
             for value in _.values():
@@ -1220,7 +1227,9 @@ class Table(Flowable):
             if er>=n: er = n-1
             self._addCommand((c[0],)+((sc, sr), (ec, er))+tuple(c[3:]))
 
-    def _cr_1_1(self,n,repeatRows, cmds,_srflMode=False):
+    def _cr_1_1(self, n, nRows, repeatRows, cmds, _srflMode=False):
+        nrr = len(repeatRows)
+        rrS = set(repeatRows)
         for c in cmds:
             (sc,sr), (ec,er) = c[1:3]
             if sr in ('splitfirst','splitlast'):
@@ -1228,22 +1237,27 @@ class Table(Flowable):
                 self._addCommand(c)
                 if sr=='splitlast': continue
                 sr = er = n
-            if sr>=0 and sr>=repeatRows and sr<n and er>=0 and er<n: continue
-            if sr>=repeatRows and sr<n: sr=repeatRows
-            elif sr>=repeatRows and sr>=n: sr=sr+repeatRows-n
-            if er>=repeatRows and er<n: er=repeatRows
-            elif er>=repeatRows and er>=n: er=er+repeatRows-n
+            if sr<0: sr += nRows
+            if er<0: er += nRows
+            cS = set(xrange(sr,er+1)) & rrS
+            if cS:
+                #it's a repeat row
+                cS = list(cS)
+                self._addCommand((c[0],)+((sc, repeatRows.index(min(cS))), (ec, repeatRows.index(max(cS))))+tuple(c[3:]))
+            if er<n: continue
+            sr = max(sr-n,0)+nrr
+            er = max(er-n,0)+nrr
             self._addCommand((c[0],)+((sc, sr), (ec, er))+tuple(c[3:]))
         sr = self._rowSplitRange
         if sr:
             sr, er = sr
-            if sr>=0 and sr>=repeatRows and sr<n and er>=0 and er<n:
+            if sr<0: sr += nRows
+            if er<0: er += nRows
+            if er<n:
                 self._rowSplitRange = None
             else:
-                if sr>=repeatRows and sr<n: sr=repeatRows
-                elif sr>=repeatRows and sr>=n: sr=sr+repeatRows-n
-                if er>=repeatRows and er<n: er=repeatRows
-                elif er>=repeatRows and er>=n: er=er+repeatRows-n
+                sr = max(sr-n,0)+nrr
+                er = max(er-n,0)+nrr
                 self._rowSplitRange = sr,er
 
     def _cr_1_0(self,n,cmds,_srflMode=False):
@@ -1328,7 +1342,7 @@ class Table(Flowable):
                         A.append(('INNERGRID',(sc,sr), (ec,er), weight, color, cap, dash, join, count, space))
                 else:
                     A.append((op,(sc,sr), (ec,er), weight, color, cap, dash, join, count, space))
-            elif op in ('INNERGRID','LINEABOVE'):
+            elif op == 'INNERGRID':
                 if sr<n and er>=n:
                     A.append(('LINEBELOW',(sc,n-1), (ec,n-1), weight, color, cap, dash, join, count, space))
                     A.append(('LINEABOVE',(sc,n), (ec,n), weight, color, cap, dash, join, count, space))
@@ -1336,7 +1350,7 @@ class Table(Flowable):
             elif op == 'LINEBELOW':
                 if sr<n and er>=(n-1):
                     A.append(('LINEABOVE',(sc,n), (ec,n), weight, color, cap, dash, join, count, space))
-                A.append((op,(sc,sr), (ec,er), weight, color))
+                A.append((op,(sc,sr), (ec,er), weight, color, cap, dash, join, count, space))
             elif op == 'LINEABOVE':
                 if sr<=n and er>=n:
                     A.append(('LINEBELOW',(sc,n-1), (ec,n-1), weight, color, cap, dash, join, count, space))
@@ -1358,29 +1372,28 @@ class Table(Flowable):
         if repeatRows:
             if isinstance(repeatRows,int):
                 iRows = data[:repeatRows]
-                nRepeatRows = repeatRows
                 iRowH = splitH[:repeatRows]
                 iCS = self._cellStyles[:repeatRows]
+                repeatRows = list(xrange(repeatRows))
             else:
                 #we have a list of repeated rows eg (1,3)
                 repeatRows = list(sorted(repeatRows))
                 iRows = [data[i] for i in repeatRows]
-                nRepeatRows = len(repeatRows)
                 iRowH = [splitH[i] for i in repeatRows]
                 iCS = [self._cellStyles[i] for i in repeatRows]
             R1 = self.__class__(iRows+data[n:],colWidths=self._colWidths,
                     rowHeights=iRowH+splitH[n:],
-                    repeatRows=nRepeatRows, repeatCols=repeatCols,
+                    repeatRows=len(repeatRows), repeatCols=repeatCols,
                     splitByRow=splitByRow, normalizedData=1,
                     cellStyles=iCS+self._cellStyles[n:],
                     ident=ident,
                     spaceAfter=getattr(self,'spaceAfter',None),
                     longTableOptimize=lto,
                     )
-            R1._cr_1_1(n,nRepeatRows,A)
-            R1._cr_1_1(n,nRepeatRows,self._bkgrndcmds,_srflMode=True)
-            R1._cr_1_1(n,nRepeatRows,self._spanCmds)
-            R1._cr_1_1(n,nRepeatRows,self._nosplitCmds)
+            R1._cr_1_1(n,nrows,repeatRows,A) #linecommands
+            R1._cr_1_1(n,nrows,repeatRows,self._bkgrndcmds,_srflMode=True)
+            R1._cr_1_1(n,nrows,repeatRows,self._spanCmds)
+            R1._cr_1_1(n,nrows,repeatRows,self._nosplitCmds)
         else:
             #R1 = slelf.__class__(data[n:], self._argW, self._argH[n:],
             R1 = self.__class__(data[n:], colWidths=self._colWidths, rowHeights=self._argH[n:],
